/*
01/01/94 Added beep functions
01/01/94 Add relay to disable ZIP power
07/08/94 Add Please wait message to checksum section
09/06/94 simplified chechsum calculation in section3
                                ______   ______
                                |     |_|     |                                 
               P1.0  -| 1        40 |-  VCC
                        P1.1  -| 2        39 |-  P0.0/AD0 Ŀ
                        P1.2  -| 3        38 |-  P0.1/AD1        
                        P1.3  -| 4        37 |-  P0.2/AD2        
          (P1.0-1.7)     P1.4  -| 5        36 |-  P0.3/AD3    (P0.0-0.7)
                        P1.5  -| 6        35 |-  P0.4/AD4        
                        P1.6  -| 7        34 |-  P0.5/AD5        
               P1.7  -| 8        33 |-  P0.6/AD6        
          (+5V )          RST  -| 9        32 |-  P0.7/AD7 
          (P3.0)     RXD/P3.0  -| 10       31 |-  VPP/EA
          (P3.1)     TXD/P3.1  -| 11       30 |-  *PROG/ALE   (P3.2)
                   *INT0/P3.2  -| 12       29 |-  *PSEN       ( 0V )
                   *INT1/P3.3  -| 13       28 |-  P2.7/A15    (P3.3) OE
                      T0/P3.4  -| 14       27 |-  P2.6/A14    ( 0V )
                      T1/P3.5  -| 15       26 |-  P2.5/A13 Ŀ
                     *WR/P3.6  -| 16       25 |-  P2.4/A12        
                     *RD/P3.7  -| 17       24 |-  P2.3/A11    (P2.8-2.13)
                        XTAL2  -| 18       23 |-  P2.2/A10        
                        XTAL1  -| 19       22 |-  P2.1/A9         
                          VSS  -| 20       21 |-  P2.0/A8  
                                |_____________|

    P3.0 RXD
    P3.1 TXD
    P3.2 ALE/PGM   PIN 30  (25 PROG PULSES)
    P3.3 OE        PIN 28
    P3.4 VPP CON   PIN 31
    P3.5 PWR RELAY
    P3.6
    P3.7 LCD ENABLE
*/
#include <8051io.h>
#include <8051bit.h>
#include <8051reg.h>
#include "8051sfb.h"

#define	XOFF	('S'-0x40)
#define	XON		('Q'-0x40)
#define	ESC		0x1B

char Version[] = "V1.0N"                 /* Current Version  */

register char CharH, CharL, temp2;       /* Holds frame timer results.*/
register unsigned int bitrate;           /* Holds final bit rate.     */
register unsigned int min, max ,temp, user;
register char saveenv[3];

main()
{
   char charin;
   min = 0x0000;
   max = 0x1FFF;

   autobaud();
   switch(_setjmp(saveenv)) {
       case 1 : putstr("\n\tInvalid command!\n"); beep(); break;
       case 2 : putstr("\n\tInvalid speed\n"); beep(); break;
       case 3 : putstr("\007\n\tCompare Error !\n\007") ;beep(); break;
       case 4 : putstr("\n\tAborted!\n"); beep(); break;
       }
/* intialize control signals to safe state */
   menu();

   while((charin=toupper(getchr())) !='Q')          /* enter Q to exit program */
          switch(charin)
   {
   case '1': section_1(); menu(); break;
   case '2': section_2(); menu(); break;
   case '3': section_3(); menu(); break;
   case '4': section_4(); menu(); break;
   case '5': section_5(); menu(); break;
   case '6': section_6(); menu(); break;
   case '7': section_7(); menu(); break;
   case '8': section_8(); menu(); break;
   case '9': section_9(); menu(); break;

   default : _longjmp(saveenv, 1);
    }
}
menu()
{
  putstr("\n\t͸\n");
    printf("\t 8751 Programmer %s  %5u bps  EPROM %5u - %5u bytes \n",Version ,bitrate,min ,max+1);
    putstr("\t͵\n");
    putstr("\t1 Program 8751 QP  4 Help              7 Set EPROM size    \n");
    putstr("\t2 Read 8751        5 Autobaud          8 8751 Checksum     \n");
    putstr("\t3 Upload Hex       6 Compare W/Upload  9 File Checksum     \n");
    putstr("\t;\n");
}

/* PROGRAM 8751 QP */
section_1()
{
    unsigned int  f_add;
    setbit(P3.2);                   /* ALE/P */
    setbit(P3.3);                   /* P2.7  */
    setbit(P3.4);                   /* ENABLE 12.5V */
    if(bitrate != 2400){
        putstr("\n\007\tChange to speed to 2400 bps, then press enter.");
        autobaud();
        }
    if(bitrate != 2400)return;

    putstr("\n\t87C51FA-1 12.5V");
    putstr("\n\t8753H     21.0V");

    putstr("\n\tStart your raw ascii upload\n\007");
    f_add=min;
    temp2=0;
    while(f_add <= max ){
        P2=f_add >> 8;                          /* h_add */
        P1=f_add;                               /* l_add */
        P0=sidchk();
        if(temp2)break;

asm {
*
* Program 25 100us pulses
*
        MOV     R6,#25          * Load 25 cycle counter
PULSE   MOV     R7,#46          * 100 us counter
        CLR     P3.2            * Lower PGM* LINE
DLY     DJNZ    R7,DLY          * Wait for time out
        SETB    P3.2            * Raise PGM*
        NOP
        NOP                     * 10 us delay between pulses
        NOP
        NOP
        NOP
        DJNZ    R6,PULSE       * Program verify this byte
      }
    ++f_add;
    }
        TmrDelay100US(8000);            /* allow 8 secs to setup upload */
        putstr("\n\tProgramming Completed \n\007");
 }

/* Read 8751   */
section_2()
{
        unsigned int  f_add;
        unsigned char data, l_add, h_add,charin;
        unsigned int  i;

        P0=0xFF;                /* Enable port for input */
        setbit(P3.2);                   /* ALE/P */
        clrbit(P3.3);                   /* P2.7  */
        TmrDelay100US(50);
        f_add=min;
      while(f_add <= max ){
        printf("\n%04x ", f_add);

        for(i=0;i < 16; ++i){

        h_add=f_add >> 8;                       /* also address/16   */
        l_add=f_add;
        P2=h_add;
        P1=l_add;
        TmrDelay100US(1);
        data=P0;
            printf(" %02x", data);
            ++f_add;
        charin=chkch();

        switch(charin){
        case XOFF: while(chkch() !=XON); break; /* Flow control */
        case  ESC: _longjmp(saveenv, 4); break;
        }
      }
            f_add -= 16;
            putstr("  ");

        for(i=0;i < 16; ++i){

        h_add=f_add >> 8;                       /* also address/16   */
        l_add=f_add;
        P2=h_add;
        P1=l_add;
        TmrDelay100US(1);
        data=P0;
            if((data > 31)&&(data < 128))       /* display only printable characters */
            printf("%c",data);
            else putchr('.');                   /*change to putchr */
            ++f_add;

        charin=chkch();

        switch(charin){
        case XOFF: while(chkch() !=XON); break;  /* Flow control */
        case  ESC: _longjmp(saveenv, 4); break;
        }
        }
    }
}

/* Upload Hex     */
section_3()
{
        unsigned int  f_add;
        unsigned char data, l_add, h_add,charin, checksum,type,count;
        unsigned int  i;

        P0=0xFF;                /* Enable port for input */
        setbit(P3.2);               /* ALE/P */
        clrbit(P3.3);               /* P2.7  */
        TmrDelay100US(50);
        f_add=min; checksum=0;count=16;type=0;
      while(f_add <= max ){
        h_add=f_add >> 8;                       /* also address/16   */
        l_add=f_add;
        printf("\n:%02x%02x%02x%02x",count, h_add, l_add, type);
            checksum += count + h_add + l_add + type;  
        for(i=0;i < 16; ++i){

         h_add=f_add >> 8;                       /* also address/16   */
         l_add=f_add  & 0x00FF;
         P2=h_add;
         P1=l_add;
         TmrDelay100US(1);
         data=P0;
         checksum += data;
            printf("%02x", data);
            ++f_add;
        charin=chkch();

        switch(charin){
        case XOFF: while(chkch() !=XON); break; /* Flow control */
        case  ESC: _longjmp(saveenv, 4); break;
        }
      }                                         /* 2 s complement */
        printf("%02x",~checksum+1);
            checksum=0;
    }
        putstr("\n:00000001FF\n");
        getchr();
}

/* Program 8751 50ms */
section_4()
{
    putstr("\n\tProgramming Help\n");
/*  unsigned int  f_add;
    unsigned char h_add, l_add;

    setbit(P3.2);
    setbit(P3.3);
    setbit(P3.4);
    putstr("\t50ms Programming pulse\n");
    putstr("\007\tStart your raw ascii upload\n\007 ");
    f_add=min;
    while(f_add <= max ){
        h_add=f_add >> 8;
        l_add=f_add;
        P2=h_add;
        P1=l_add;
        P0=getchr();
    TmrDelay100US(1);
    clrbit(P3.2);
    TmrDelay100US(495);
    setbit(P3.2);
    ++f_add;
    }
        TmrDelay100US(8000);
        putstr("\tProgramming Completed \n\007");
 */
 }
/* Autobaud       */
section_5()
{
    putstr("\n\x07\tChange speed now, then press any key.\n");
    autobaud();
}
/* Compare W/Upload */
section_6()
{
        unsigned int  f_add;
        unsigned char data, compare, l_add, h_add;

        P0=0xFF;                /* Enable port for input */
        setbit(P3.2);                   /* ALE/P */
        clrbit(P3.3);                   /* P2.7  */
        putstr("\007\tStart your raw ascii upload now\n\007");
        f_add=min;
      while(f_add <= max ){
        h_add=f_add >> 8;                       /* also address/16   */
        l_add=f_add;
        P2=h_add;
        P1=l_add;
        TmrDelay100US(1);
            data=P0;
            compare=getchr();
            if(data != compare){
                printf("\tError at address %04x  %02x %02x", f_add, data, compare);
                putchr(XOFF);
                _longjmp(saveenv,3);
            }
        ++f_add;
      }
        TmrDelay100US(8000);            /* allow 8 secs to setup upload */
        putstr("\tCompare Ok \n\007");
    }

/* Set EPROM size */
section_7()
{
   char charin;
   putstr("\tEnter starting address :\n");
   putstr("\t0=0000h\n");
   putstr("\t1=1000h\n");
   putstr("\t2=2000h\n");
   putstr("\t4=4000h\n");
   putstr("\t8=8000h\n");
   putstr("\tA=0100h\n");
   putstr("\tB=0200h\n");

  charin=getchr();
          switch(charin)
   {
   case '0': min = 0x0000; break;
   case '1': min = 0x1000; break;
   case '2': min = 0x2000; break;
   case '4': min = 0x4000; break;
   case '8': min = 0x8000; break;
   case 'A': min = 0x0100; break;
   case 'B': min = 0x0200; break;
   case ESC: _longjmp(saveenv, 4); break;
   default : min = 0x0000;
    }
   putstr("\tEnter size of Eprom\n");
   putstr("\t1=1000h\n");
   putstr("\t2=2000h\n");
   putstr("\t4=4000h\n");
   putstr("\t8=8000h\n");
   putstr("\t0=0100h\n");
   printf("\t9=%04xh\n",user);

  charin=getchr();
          switch(charin)
   {
   case '1': max = min + 0x0FFF; break;
   case '2': max = min + 0x1FFF; break;
   case '4': max = min + 0x3FFF; break;
   case '8': max = min + 0x7FFF; break;
   case '0': max = min + 0x00FF; break;
   case '9': max = user;         break;
   case ESC: _longjmp(saveenv, 4); break;
   default : max = min + 0x1FFF;
    }
}
section_8()
{
        unsigned int  f_add;
        unsigned char data, l_add, h_add;
        unsigned int checksum;

        checksum = 0;
        putstr("\n\007\tCalculating Checksum Please wait . . .");
        P0=0xFF;                /* Enable port for input */
        setbit(P3.2);                   /* ALE/P */
        clrbit(P3.3);                   /* P2.7  */
        TmrDelay100US(50);
        f_add=min;
      while(f_add <= max ){
        h_add=f_add >> 8;                       /* also address/16   */
        l_add=f_add;
        P2=h_add;
        P1=l_add;
        TmrDelay100US(1);
        data=P0;
        checksum+=data;
            ++f_add;
      }
        printf("\n\tChecksum = %04x",checksum);
}
section_9()
{
        unsigned int  f_add;
        unsigned char data;
        unsigned int checksum;
        putstr("\n\tStart your raw ascii upload\n\007");
        checksum = 0;
        f_add=min;
        temp2=0;
      while(f_add <= max ){
        data=sidchk();
        if(temp2)break;
        checksum+=data;
            ++f_add;
      }
printf("\n\tChecksum = %04x\t Bytes= %04u Bytes Hex= %04x\n",checksum, f_add, f_add);
        user=f_add-1;
}
/*---------------------------------------------------------------------------*/
/* Delay for a fixed number of 100 microsecond ticks                         */
/* Zero NumUS is special-cased to bail out                                   */

TmrDelay100US(Num100US)
unsigned Num100US;
{
   if (!Num100US) {                    /* bail out for zero input            */
      return;
   }
   do {
 asm {
      MOV   B,#43
Tmr?1 DJNZ  B,Tmr?1
 }
   } while (--Num100US);
}
double_bps()
{
    PCON |= 0x80;                   /* set PCON.7       */
}
beep()
{
    putchr('\x07');
}

/* AUTOBAUD ROUTINE */
autobaud()
{
        switch(auto2())
    {
    /* 8051 amonmaly doubling 19200 actually set bit rate to 57600 */
    case 1 : serinit(19200);double_bps(); bitrate =57600;break;
    case 2 : serinit(9600); double_bps(); bitrate =19200;break;
    case 3 : serinit(4800); double_bps(); bitrate = 9600;break;
    case 4 : serinit(2400); double_bps(); bitrate = 4800;break;
    case 5 : serinit(1200); double_bps(); bitrate = 2400;break;
    case 6 : serinit(600);  double_bps(); bitrate = 1200;break;
    case 8 : serinit(150);  double_bps(); bitrate =  300;break;
    default :serinit(4800); double_bps(); bitrate = 9600;break;
    }
}
auto2()
{
    TmrDelay100US(4000);
asm
{
*   AutoBaud Rate Detect Routine.
*   Attempts to detect baud rate from first received character, by measuring
*   the length of the character. Some characters may not work properly,
*   primarily those that end with more than 3 (4?) ones in a row.
*   Returns with ACC = baud rate pointer.

A07       MOV   TMOD,#01        *Initialize timer 0 (UART baud rate timer).
          MOV   TH0,#00         *Put timer 0 in 16-bit counter mode.
          MOV   TL0,#00
          MOV   TCON,#00

          MOV   CharH,#00       *Initialize timer result.
          MOV   CharL,#00
A19       JB    P3.0,A19        *Wait for serial start bit.
          SETB  TCON.4          *Start timer.

A1E       JB    TCON.5,A32      *Check for timer overflow.
          JNB   P3.0,A1E        *Check for a rising edge on serial data.

          MOV   CharH,TH0       *Capture timer value at this serial edge.
          MOV   CharL,TL0

A2A       JB    TCON.5,A32      *Check for timer overflow.
          JB    P3.0,A2A        *Check for falling edge on serial data.
          SJMP  A1E             *Go back and repeat sampling.

A32       CLR   TCON.4          *Maximum sample time has expired, check result.
          CLR   TCON.5          *Begin by stopping timer and clearing flag.

          MOV   bitrate,#19     *Set up table pointers.
CmpLoop   MOV   A,bitrate
          MOV   DPTR,#CmpTable
          MOVC  A,[A+DPTR]      *Get a table entry for comparison.
          DEC   bitrate
          CJNE  A,CharH,Cmp1    *Check result range.
          SJMP  CmpLow          *High byte table = timed value, check low byte.
Cmp1      JC    CmpMatch        *A match if table value is < timed value.
          DJNZ  bitrate,CmpLoop *Check for end of comparison table.
          SJMP  CmpMatch

CmpLow    MOV   A,bitrate
          MOVC  A,[A+DPTR]      *Get a table entry for comparison.
          CJNE  A,CharL,Cmp2    *Check result range.
          SETB  C               *Match if equal.
Cmp2      JC    CmpMatch        *C set if A < low byte of result.
          DJNZ  bitrate,CmpLoop *Check for end of comparison table.

CmpMatch  MOV   A,bitrate       *Comparison complete,
          CLR   C               *get final baud rate index,
          RRC   A
          MOV   bitrate,A       *and save.
          RET
* Compare table holds timer values for the transition points of the accepted
* baud rates. Entries are LSB, MSB. These values are for 11,0592 MHZ operation.

CmpTable
          DB    $40,$00          ;0 - out of range, value too low.
          DB    $78,$00          ;1 - 38400 baud.
          DB    $F0,$00          ;2 - 19200 baud.
          DB    $E0,$01          ;3 -  9600 baud.
          DB    $C0,$03          ;4 -  4800 baud.
          DB    $80,$07          ;5 -  2400 baud.
          DB    $00,$0F          ;6 -  1200 baud.
          DB    $00,$1E          ;7 -   600 baud.
          DB    $00,$3C          ;8 -   300 baud.
          DB    $00,$80          ;9 - out of range, value too high.
}
}
sidchk()
{
    unsigned int temp, count;
    count =1;
    while((temp=chkchr()) == EOF){
        if(count == 0xffff){
        putstr("\n\t\007Timeout ");
        temp2=1;
        return temp;
        }
    ++count;
    }
    return temp;
}
